@@ -1,7 +1,9 @@
 /**
  * parsetrigrams.cpp
  *
- * Parse a set of trigram files into a QHash, and serialize to stdout.
+ * Parse a set of trigram files into a QMap, and serialize to stdout.
+ * Note: we allow this data to be read into QHash. We use QMap here
+ * to get deterministic output from run to run.
  *
  * SPDX-FileCopyrightText: 2006 Jacob Rideout <kde@jacobrideout.net>
  *
@@ -11,7 +13,7 @@
 #include <QDataStream>
 #include <QDir>
 #include <QFile>
-#include <QHash>
+#include <QMap>
 #include <QRegularExpression>
 #include <QString>
 #include <QTextStream>
@@ -29,19 +31,32 @@ int main(int argc, char **argv)
     QString path = QLatin1String(argv[1]);
     QDir td(path);
 
-    QHash<QString, QHash<QString, int>> models;
+    /*
+     * We use QMap (instead of QHash) here to get deterministic output
+     * from run to run.
+     */
+    QMap<QString, QMap<QString, int>> models;
 
     const QRegularExpression rx(QStringLiteral("(?:.{3})\\s+(.*)"));
-    for (const QString &fname : td.entryList(QDir::Files)) {
+    const QStringList files = td.entryList(QDir::Files);
+    for (const QString &fname : files) {
         QFile fin(td.filePath(fname));
         fin.open(QFile::ReadOnly | QFile::Text);
         QTextStream stream(&fin);
+
+        // Not needed with Qt6, UTF-8 is the default
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         stream.setCodec("UTF-8");
+#endif
         while (!stream.atEnd()) {
             QString line = stream.readLine();
             const QRegularExpressionMatch match = rx.match(line);
             if (match.hasMatch()) {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+                models[fname][line.left(3)] = match.capturedView(1).toInt();
+#else
                 models[fname][line.left(3)] = match.capturedRef(1).toInt();
+#endif
             }
         }
     }
