@@ -10,25 +10,27 @@
 
 #include "highlighter.h"
 
-#include "speller.h"
-#include "loader_p.h"
-#include "tokenizer_p.h"
-#include "settings_p.h"
 #include "languagefilter_p.h"
+#include "loader_p.h"
+#include "settingsimpl_p.h"
+#include "speller.h"
+#include "tokenizer_p.h"
 
 #include "ui_debug.h"
-#include <QTextEdit>
-#include <QTextCharFormat>
-#include <QTimer>
+
 #include <QColor>
-#include <QHash>
-#include <QTextCursor>
 #include <QEvent>
+#include <QHash>
 #include <QKeyEvent>
 #include <QMetaMethod>
 #include <QPlainTextEdit>
+#include <QTextCharFormat>
+#include <QTextCursor>
+#include <QTextEdit>
+#include <QTimer>
 
-namespace Sonnet {
+namespace Sonnet
+{
 // Cache of previously-determined languages (when using AutoDetectLanguage)
 // There is one such cache per block (paragraph)
 class LanguageCache : public QTextBlockUserData
@@ -45,7 +47,7 @@ public:
         it.toBack();
         while (it.hasPrevious()) {
             it.previous();
-            if (it.key().first+it.key().second >= pos) {
+            if (it.key().first + it.key().second >= pos) {
                 it.remove();
             } else {
                 break;
@@ -79,6 +81,7 @@ public:
         tokenizer = new WordTokenizer();
         active = true;
         automatic = false;
+        autoDetectLanguageDisabled = false;
         connected = false;
         wordCount = 0;
         errorCount = 0;
@@ -93,8 +96,7 @@ public:
         spellchecker = new Sonnet::Speller();
         spellCheckerFound = spellchecker->isValid();
         rehighlightRequest = new QTimer(q);
-        q->connect(rehighlightRequest, SIGNAL(timeout()),
-                   q, SLOT(slotRehighlight()));
+        q->connect(rehighlightRequest, &QTimer::timeout, q, &Highlighter::slotRehighlight);
 
         if (!spellCheckerFound) {
             return;
@@ -118,16 +120,17 @@ public:
     QPlainTextEdit *plainTextEdit = nullptr;
     bool active;
     bool automatic;
+    bool autoDetectLanguageDisabled;
     bool completeRehighlightRequired;
     bool intraWordEditing;
-    bool spellCheckerFound; //cached d->dict->isValid() value
+    bool spellCheckerFound; // cached d->dict->isValid() value
     bool connected;
     int disablePercentage = 0;
     int disableWordCount = 0;
     int wordCount, errorCount;
     QTimer *rehighlightRequest = nullptr;
     QColor spellColor;
-    Highlighter *q;
+    Highlighter *const q;
 };
 
 HighlighterPrivate::~HighlighterPrivate()
@@ -173,7 +176,7 @@ void Highlighter::slotRehighlight()
         d->errorCount = 0;
         rehighlight();
     } else {
-        //rehighlight the current para only (undo/redo safe)
+        // rehighlight the current para only (undo/redo safe)
         QTextCursor cursor;
         if (d->textEdit) {
             cursor = d->textEdit->textCursor();
@@ -185,8 +188,8 @@ void Highlighter::slotRehighlight()
         }
         cursor.insertText(QString());
     }
-    //if (d->checksDone == d->checksRequested)
-    //d->completeRehighlightRequired = false;
+    // if (d->checksDone == d->checksRequested)
+    // d->completeRehighlightRequired = false;
     QTimer::singleShot(0, this, SLOT(slotAutoDetection()));
 }
 
@@ -195,6 +198,11 @@ bool Highlighter::automatic() const
     return d->automatic;
 }
 
+bool Highlighter::autoDetectLanguageDisabled() const
+{
+    return d->autoDetectLanguageDisabled;
+}
+
 bool Highlighter::intraWordEditing() const
 {
     return d->intraWordEditing;
@@ -217,15 +225,23 @@ void Highlighter::setAutomatic(bool automatic)
     }
 }
 
+void Highlighter::setAutoDetectLanguageDisabled(bool autoDetectDisabled)
+{
+    d->autoDetectLanguageDisabled = autoDetectDisabled;
+}
+
 void Highlighter::slotAutoDetection()
 {
     bool savedActive = d->active;
 
-    //don't disable just because 1 of 4 is misspelled.
+    // don't disable just because 1 of 4 is misspelled.
     if (d->automatic && d->wordCount >= 10) {
         // tme = Too many errors
-        bool tme = (d->errorCount >= d->disableWordCount) && (
-            d->errorCount * 100 >= d->disablePercentage * d->wordCount);
+        /* clang-format off */
+        bool tme = (d->errorCount >= d->disableWordCount)
+                   && (d->errorCount * 100 >= d->disablePercentage * d->wordCount);
+        /* clang-format on */
+
         if (d->active && tme) {
             d->active = false;
         } else if (!d->active && !tme) {
@@ -235,11 +251,12 @@ void Highlighter::slotAutoDetection()
 
     if (d->active != savedActive) {
         if (d->active) {
-            emit activeChanged(tr("As-you-type spell checking enabled."));
+            Q_EMIT activeChanged(tr("As-you-type spell checking enabled."));
         } else {
             qCDebug(SONNET_LOG_UI) << "Sonnet: Disabling spell checking, too many errors";
-            emit activeChanged(tr("Too many misspelled words. "
-                                  "As-you-type spell checking disabled."));
+            Q_EMIT activeChanged(
+                tr("Too many misspelled words. "
+                   "As-you-type spell checking disabled."));
         }
 
         d->completeRehighlightRequired = true;
@@ -257,9 +274,9 @@ void Highlighter::setActive(bool active)
     rehighlight();
 
     if (d->active) {
-        emit activeChanged(tr("As-you-type spell checking enabled."));
+        Q_EMIT activeChanged(tr("As-you-type spell checking enabled."));
     } else {
-        emit activeChanged(tr("As-you-type spell checking disabled."));
+        Q_EMIT activeChanged(tr("As-you-type spell checking disabled."));
     }
 }
 
@@ -276,7 +293,7 @@ void Highlighter::contentsChange(int pos, int add, int rem)
     do {
         LanguageCache *cache = dynamic_cast<LanguageCache *>(block.userData());
         if (cache) {
-            cache->invalidate(pos-block.position());
+            cache->invalidate(pos - block.position());
         }
         block = block.next();
     } while (block.isValid() && block < lastBlock);
@@ -299,8 +316,7 @@ void Highlighter::highlightBlock(const QString &text)
     }
 
     if (!d->connected) {
-        connect(document(), SIGNAL(contentsChange(int,int,int)),
-                SLOT(contentsChange(int,int,int)));
+        connect(document(), &QTextDocument::contentsChange, this, &Highlighter::contentsChange);
         d->connected = true;
     }
     QTextCursor cursor;
@@ -313,8 +329,8 @@ void Highlighter::highlightBlock(const QString &text)
 
     const int lengthPosition = text.length() - 1;
 
-    if (index != lengthPosition
-        || (lengthPosition > 0 && !text[lengthPosition-1].isLetter())) {
+    if (index != lengthPosition //
+        || (lengthPosition > 0 && !text[lengthPosition - 1].isLetter())) {
         d->languageFilter->setBuffer(text);
 
         LanguageCache *cache = dynamic_cast<LanguageCache *>(currentBlockUserData());
@@ -325,8 +341,8 @@ void Highlighter::highlightBlock(const QString &text)
 
         const bool autodetectLanguage = d->spellchecker->testAttribute(Speller::AutoDetectLanguage);
         while (d->languageFilter->hasNext()) {
-            QStringRef sentence = d->languageFilter->next();
-            if (autodetectLanguage) {
+            Token sentence = d->languageFilter->next();
+            if (autodetectLanguage && !d->autoDetectLanguageDisabled) {
                 QString lang;
                 QPair<int, int> spos = QPair<int, int>(sentence.position(), sentence.length());
                 // try cache first
@@ -348,21 +364,21 @@ void Highlighter::highlightBlock(const QString &text)
             d->tokenizer->setBuffer(sentence.toString());
             int offset = sentence.position();
             while (d->tokenizer->hasNext()) {
-                QStringRef word = d->tokenizer->next();
+                Token word = d->tokenizer->next();
                 if (!d->tokenizer->isSpellcheckable()) {
                     continue;
                 }
                 ++d->wordCount;
                 if (d->spellchecker->isMisspelled(word.toString())) {
                     ++d->errorCount;
-                    setMisspelled(word.position()+offset, word.length());
+                    setMisspelled(word.position() + offset, word.length());
                 } else {
-                    unsetMisspelled(word.position()+offset, word.length());
+                    unsetMisspelled(word.position() + offset, word.length());
                 }
             }
         }
     }
-    //QTimer::singleShot( 0, this, SLOT(checkWords()) );
+    // QTimer::singleShot( 0, this, SLOT(checkWords()) );
     setCurrentBlockState(0);
 }
 
@@ -377,9 +393,7 @@ void Highlighter::setCurrentLanguage(const QString &lang)
     d->spellchecker->setLanguage(lang);
     d->spellCheckerFound = d->spellchecker->isValid();
     if (!d->spellCheckerFound) {
-        qCDebug(SONNET_LOG_UI) << "No dictionary for \""
-                               << lang
-                               << "\" staying with the current language.";
+        qCDebug(SONNET_LOG_UI) << "No dictionary for \"" << lang << "\" staying with the current language.";
         d->spellchecker->setLanguage(prevLang);
         return;
     }
@@ -411,10 +425,11 @@ bool Highlighter::eventFilter(QObject *o, QEvent *e)
     }
     if ((o == d->textEdit || o == d->plainTextEdit) && (e->type() == QEvent::KeyPress)) {
         QKeyEvent *k = static_cast<QKeyEvent *>(e);
-        //d->autoReady = true;
+        // d->autoReady = true;
         if (d->rehighlightRequest->isActive()) { // try to stay out of the users way
             d->rehighlightRequest->start(500);
         }
+        /* clang-format off */
         if (k->key() == Qt::Key_Enter
             || k->key() == Qt::Key_Return
             || k->key() == Qt::Key_Up
@@ -425,12 +440,12 @@ bool Highlighter::eventFilter(QObject *o, QEvent *e)
             || k->key() == Qt::Key_PageDown
             || k->key() == Qt::Key_Home
             || k->key() == Qt::Key_End
-            || ((k->modifiers() == Qt::ControlModifier)
-                && ((k->key() == Qt::Key_A)
-                    || (k->key() == Qt::Key_B)
-                    || (k->key() == Qt::Key_E)
-                    || (k->key() == Qt::Key_N)
-                    || (k->key() == Qt::Key_P)))) {
+            || (k->modifiers() == Qt::ControlModifier
+                && (k->key() == Qt::Key_A
+                    || k->key() == Qt::Key_B
+                    || k->key() == Qt::Key_E
+                    || k->key() == Qt::Key_N
+                    || k->key() == Qt::Key_P))) { /* clang-format on */
             if (intraWordEditing()) {
                 setIntraWordEditing(false);
                 d->completeRehighlightRequired = true;
@@ -441,15 +456,15 @@ bool Highlighter::eventFilter(QObject *o, QEvent *e)
         } else {
             setIntraWordEditing(true);
         }
-        if (k->key() == Qt::Key_Space
-            || k->key() == Qt::Key_Enter
+        if (k->key() == Qt::Key_Space //
+            || k->key() == Qt::Key_Enter //
             || k->key() == Qt::Key_Return) {
             QTimer::singleShot(0, this, SLOT(slotAutoDetection()));
         }
-    } else if (((d->textEdit && (o == d->textEdit->viewport()))
-                || (d->plainTextEdit && (o == d->plainTextEdit->viewport())))
+    } else if (((d->textEdit && (o == d->textEdit->viewport())) //
+                || (d->plainTextEdit && (o == d->plainTextEdit->viewport()))) //
                && (e->type() == QEvent::MouseButtonPress)) {
-        //d->autoReady = true;
+        // d->autoReady = true;
         if (intraWordEditing()) {
             setIntraWordEditing(false);
             d->completeRehighlightRequired = true;
